/*******************************************************

This program will test the Pololu Jrk controller library
  using direct control of the controller in Serial UART
  mode.
Robin Sharp, September 2014.

********************************************************/

// Needs:
//   SPI and Wire libraries for infrastructure
//   SD library for logging
//   MCP23017 and RGBLCDShield libraries for display
//   RTClib library for the real-time clock
//   JrkSerial library for Pololu 12v12 controller for actuator
#include <Arduino.h>
#include <SPI.h>
#include <Wire.h>
#include <Adafruit_MCP23017.h>
#include <Adafruit_RGBLCDShield.h>
#include "RTClib.h"
#include <JrkSerial.h>

// Codes to set the RGB display backlight color
#define RED    0x1
#define YELLOW 0x3
#define GREEN  0x2
#define TEAL   0x6
#define BLUE   0x4
#define VIOLET 0x5
#define WHITE  0x7

// Target value limits for the actuator (in range [0,4095]
// Thse may need adjustment to suit physical setup of system.
#define actuMIN  256        // Minimum target value for actuator
#define actuMAX  3839       // Maximum target value for actuator

// Variables *******************************************************
uint16_t   actutarget=actuMIN;     // Target for actuator (in [0;4095])
uint8_t    windowtarget=0;         // Target for window opening (in [0;100])

// Bit array indicating which buttons are pressed.
// Bits are defined in the RGBLCDShield library as follows:
//   Bit 0 (value  1) SELECT
//   Bit 1 (value  2) RIGHT
//   Bit 2 (value  4) DOWN
//   Bit 3 (value  8) UP
//   Bit 4 (value 16) LEFT
uint8_t buttons     = 0;

// Values read from the controller
// The enum identifies which result has which index.
enum {fdback,scalef,target,errsum,DCval,errfla,hflags};
int  Nitems = hflags+1;
// Index of current result for display
int item = fdback;   

// Array of headings for use in displaying results.
const char* meastitle[]=
                  {"Feedback        ",
                   "Scaled feedback ",
                   "Target          ",
                   "Error sum       ",
                   "Duty cycle      ",
                   "Error flags     ",
                   "Halt flags      "};

// Increment for target
int incr = 256;

// Flag move is true if a new target is to be set
boolean move = false;

// Objects *********************************************************
// Create an Adafruit RGB SHield object for the display of results.
//    The shield uses the I2C SCL and SDA pins. 
Adafruit_RGBLCDShield lcd = Adafruit_RGBLCDShield();

// Create a JrkSerial object for controlling the actuator
JrkSerial Actu = JrkSerial();

// Auxiliary functions *********************************************
// Function to blank out a line on the LCD display
void blankout(int lineno)
{ lcd.setCursor(0,lineno);
  lcd.print("                ");
  lcd.setCursor(0,lineno);
}




// Main code *******************************************************

void setup()
{  // Start the LCD panel.
 lcd.begin(16, 2); 
 lcd.setBacklight(WHITE);
 lcd.print("Test start      ");
 lcd.setCursor(0,1);
 lcd.print("                ");
 // As default, select display of actuator position
 item = fdback;
 // Set position to initial target
 actutarget = actuMIN;
 Actu.setPosition(actutarget);
};


void loop()
{ move = false;
  // Test LCD buttons to see if action is required
  buttons = lcd.readButtons();  // read the buttons
  // lcd.print(buttons);         // De-comment if debugging

  if (buttons) {
    if (buttons & BUTTON_UP) {
       // Change to previous item (modulo no. of items)
       item = (item + Nitems - 1) % Nitems;
    }
    if (buttons & BUTTON_DOWN) {
       // Change to following item (modulo no. of items)
       item = (item + 1) % Nitems;
    }
    if (buttons & BUTTON_SELECT) {
	// Go on to the next target.
        move = true;
    }
  }
  lcd.setCursor(0,0);
  
  // If move is false, just display value of selected item.
  if (!move) {
    lcd.print(meastitle[item]);
    blankout(1);
    switch ( item )
      { case fdback:
          lcd.print( Actu.getVariable(JrkSerial::feedback) );
	  break;
        case scalef:
          lcd.print( Actu.getVariable(JrkSerial::sfeedback) );
          break;
        case target:
          lcd.print( Actu.getVariable(JrkSerial::input) );
          break;
        case errsum:
          lcd.print( Actu.getVariable(JrkSerial::errorsum) );
          break;
        case DCval :
          lcd.print( Actu.getVariable(JrkSerial::DCvalue) );
          break;
        case errfla:
          lcd.print( Actu.getErrors(), BIN );
          break;
        case hflags:
          lcd.print( Actu.motorOff(), BIN );
          break;
      }
    }
  else 
    { // If move is true, set new target
      lcd.print("Move target to: ");
      actutarget += incr;
      if ((actutarget > actuMAX) || (actutarget < actuMIN))
        { incr = -incr; 
          actutarget += 2*incr; 
        };
      Actu.setPosition(actutarget);
      blankout(1);
      lcd.print(actutarget);
    };
  delay(200);

  //  lcd.print("Stopping.      ");
  //  delay(5000);
  // exit(1);
}      
  
